(*
 * Copyright (C) 2004 Damien Pous
 * 
 * This file is part of GcPan.
 * 
 * GcPan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * GcPan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GcPan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Temporary limited type-checking *)

open Ast

let error = failwith

(* Check variables (kind/binding), and return the free names *)
let free_names p = 
  (*
    `N  : name
    `P  : path
    `NP : name or path
  *)
  let unify s = function
    |  x,  y when x=y -> x
    | `NP, x | x, `NP -> x
    | _               -> error "type error: N / P (%s)" s
  in
  let r = ref [] in
  let e = Hashtbl.create 5 in
  let decl t x = Hashtbl.add e x t in
  let use  t x = 
    let t' = 
      try Hashtbl.find e x 
      with 
	| Not_found when t = `P -> error "unbound variable (%s)" x
	| Not_found -> r:=x:: !r; `N
    in 
      Hashtbl.replace e x (unify x (t, t'))
  in
  let free x = Hashtbl.remove e x in
  let checksend = 
    let rec aux = function
      | Name v    ::q -> use `P v; aux q
      | Capa(_,v) ::q -> use `N v; aux q
      | _ -> ()
    in function
      | [Name v] -> use `NP v
      | p -> aux p
  in
  let rec check1 = function
    | Amb (v,p)
    | Amb'(_,v,p)
    | Pref(Capa(_,v),p) -> use `N v; check p
    | Pref(Name v,p)    -> use `P v; check p
    | Send pl           -> List.iter checksend pl
    | Rec (v,p)         -> check p
    | Nu  (v,p)         -> decl `N v; check p; free v
    | Rcv (vl,p)        -> List.iter (decl `NP) vl; check p; List.iter free vl
    | _                 -> ()
  and check p = List.iter check1 p
  in
    check p; !r

