(*
 * Copyright (C) 2004 Damien Pous
 * 
 * This file is part of GcPan.
 * 
 * GcPan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * GcPan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GcPan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Generic bi-matcher *)

(* a matcher is a pair of lists, <L,R> 
 * such that at most one is non-empty
 * trying to add a element l to L when R is not empty 
 * return a element of R (without adding l)
 *
 * a named-matcher is a collection of matchers, indexed by names
 *)

type 'c formatter = (Format.formatter -> 'c -> unit)

module type CONV = sig
  include GenComm.CONV
  val format: 'c formatter -> 'c t formatter
end

module type MATCHER = 
sig
  type 'c l
  type 'c r
  type 'c lr

  include CONV
  type 'c tlr = 'c t * 'c lr list

  val add_left : 'c tlr -> 'c l -> 'c tlr
  val add_right: 'c tlr -> 'c r -> 'c tlr
  val add:       'c tlr -> 'c t -> 'c tlr

  val get_left:  'c t -> 'c l list
  val get_right: 'c t -> 'c r list

  val empty: 'c t
  val is_empty: 'c t -> bool
end

module Simple(L: CONV)(R: CONV)(U: sig type 'c t val mix: 'c L.t -> 'c R.t -> 'c t end):
  MATCHER with type 'c l  = 'c L.t
	   and type 'c r  = 'c R.t
	   and type 'c lr = 'c U.t

module Named(M: MATCHER): 
  MATCHER with type 'c l  = string * 'c M.l
	   and type 'c r  = string * 'c M.r
	   and type 'c lr = 'c M.lr

module SimpleNamed(L: CONV)(R: CONV)(U: sig type 'c t val mix: 'c L.t -> 'c R.t -> 'c t end): 
  MATCHER with type 'c l  = string * 'c L.t
	   and type 'c r  = string * 'c R.t
	   and type 'c lr = 'c U.t

