(*
 * Copyright (C) 2004 Damien Pous
 * 
 * This file is part of GcPan.
 * 
 * GcPan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * GcPan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GcPan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* fresh name generator *)

module type T = sig
  type t
  val fresh: unit -> t
  val free: t -> unit
  val to_string: t -> string
  val format: Format.formatter -> t -> unit
end

module Make(M: sig val base: string end): T = struct
  type t = int
  let n = ref 0
  let l = ref []
  let fresh () = 
    match !l with
      | [] -> incr n; !n
      | x::q -> l:=q; x
  let insert x = 
    let rec aux acc = function
      | y::q when x>y -> aux (y::acc) q
      | l  -> List.rev_append (x::acc) l
    in aux []
  let free x =
    if x= !n then decr n 
    else l := insert x !l
(*   let free _ = () *)
  let to_string i = M.base^string_of_int i
  let format f i = Format.pp_print_string f (to_string i)
end
