(*
 * Copyright (C) 2004 Damien Pous
 * 
 * This file is part of GcPan.
 * 
 * GcPan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * GcPan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GcPan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Parsing *)

open Ast


(* Parse the given string *)
val from_string:  string -> process

(* Parse the given file *)
val from_file:    string -> process

(* Parse the given channel *)
val from_channel: in_channel -> process


(* Interactive parsing of a channel 
 * (with error recovery) 
 *)
val interactive: in_channel -> 
  [ `Eval      of process
  | `Add       of process
  | `AddTo     of GenComm.host * string * process
  | `Print     of process
  | `Check     of process
  | `Def       of string
  | `Cmd       of [ `Debug | `Tree | `Step of string ]
  | `SafeNames of bool
  | `PackReq   of bool
  | `RelocMode of [ `Sync | `Async | `Mixed ]
  | `Stats 
  | `Help ] Stream.t


(* Prints help (list of directives) *)
val print_help: unit -> unit

(* Prints version and licence message *)
val print_version: unit -> unit
