(*
 * Copyright (C) 2004 Damien Pous
 * 
 * This file is part of GcPan.
 * 
 * GcPan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * GcPan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GcPan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Pretty printing of SA terms *)

open Ast
open Format

let get_bang = 
  let rec rem_bang p = function
    | [] -> failwith "ARGHH: bang not found"
    | Var x::q when x=bang_var -> par p q
    | r::q -> rem_bang (r::p) q
  in
    function
      | [Pref(Capa(_,_) as c, p)] -> Pref (c, rem_bang zero p)
      | _ -> failwith "ARGHH: misplaced bang variable"

let format_string g = fun f x -> pp_print_string f (g x)

let format_cap  = format_string string_of_cap
let format_var  = format_string string_of_var
let format_host = format_string GenComm.string_of_host

let format_list sep format_elt = 
  let fmt = "%a"^^sep^^"%a" in
  let rec aux f = function
    | [] -> ()
    | [x] -> format_elt f x
    | x::q -> fprintf f fmt format_elt x aux q
  in aux

let format_expr f = function
  | Name  n    -> fprintf f "%s" n
  | Capa (c,n) -> fprintf f "%a%s" format_cap c n
  | Sleep 1    -> fprintf f "sleep"
  | Sleep i    -> fprintf f "sleep %i" i

let format_path  = format_list "."   format_expr
let format_paths = format_list ",@ " format_path
let format_names = format_list ","   pp_print_string

let rec format_process_t f = function
  | Var  x       -> format_var f x
  | Pref(e,[])   -> format_expr f e
  | Pref(e,p)    -> fprintf f "%a.%a" format_expr e format_pprocess p
  | Rcv (x,p)    -> fprintf f "(%a)%a" format_names x format_pprocess p
  | Send p       -> fprintf f "<%a>" format_paths p
  | Amb (x,[])   -> fprintf f "%s[]" x
  | Amb (x,p)    -> fprintf f "%s[@[@,%a@]]" x format_process p
  | Amb'(h,x,[]) -> fprintf f "%s@@%a[]" x format_host h
  | Amb'(h,x,p)  -> fprintf f "%s@@%a[@[@,%a@]]" x format_host h format_process p
  | Rec (x,p) when x=bang_var -> 
                    fprintf f "!%a" format_process_t (get_bang p)
  | Rec (x,p)    -> fprintf f "rec %a.@,%a" format_var x format_pprocess p
  | Nu(_,_) as p -> let l,p = uncurry_nus p in 
       fprintf f "(nu %a)%a" format_names l format_pprocess p
and format_process f = function
  | []   -> fprintf f "0"
  | [p]  -> format_process_t f p
  | p::q -> fprintf f "%a@ |@ %a" format_process_t p format_process q
and format_pprocess f = function
  | []   -> fprintf f "0"
  | [p]  -> fprintf f "@[%a@]" format_process_t p
  | p::q -> fprintf f "(@[@,%a@ |@ %a@])" format_process_t p format_process q


let print_process   p = printf "@[%a@]@." format_process   p
let print_process_t p = printf "@[%a@]@." format_process_t p

let format_list f = format_list "," f
