(*
 * Copyright (C) 2004 Damien Pous
 * 
 * This file is part of GcPan.
 * 
 * GcPan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * GcPan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GcPan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Statistics for comparing PAN and GCPAN *)


let m = Mutex.create()

let local_message = ref  0
let net_message   = ref  0
let request       = ref (0,0,0)
let completion    = ref  0
let relocation    = ref  0
let forwarder     = ref (0,0)
let out_forwarder = ref (0,0)
let gc_forwarder  = ref  0
let l_com         = ref  0
let l_in          = ref  0
let l_out         = ref  0
let l_open        = ref  0   

let reset () = 
  Mutex.lock m;
  local_message :=  0;
  net_message   :=  0; 
  request       := (0,0,0);
  completion    :=  0;
  relocation    :=  0;
  forwarder     := (0,0);
  out_forwarder := (0,0);
  gc_forwarder  :=  0;
  l_com         :=  0;
  l_in          :=  0;
  l_out         :=  0;
  l_open        :=  0;
  Mutex.unlock m

let av (x,n) = float_of_int x /. float_of_int n

let print () =
  Mutex.lock m;
  let (j,p,r) = !request in
  let (na,a), (nb,b) = !forwarder, !out_forwarder in
    Printf.printf "forwarders       : %d + %d - %d = %d\n" a b !gc_forwarder (a+b- !gc_forwarder);
    Printf.printf "local messages   : %d\n" !local_message  ;
    Printf.printf "network messages : %d\n" !net_message    ;
    Printf.printf "requests         : %d\n" r               ;
    Printf.printf "completions      : %d\n" !completion     ;
    Printf.printf "relocations      : %d\n" !relocation     ;
    Printf.printf "local_com        : %d\n" !l_com          ;
    Printf.printf "local_in         : %d\n" !l_in           ;
    Printf.printf "local_out        : %d\n" !l_out          ;
    Printf.printf "local_open       : %d\n" !l_open         ;
    Printf.printf "average forwarder lifetime      : %.2f\n" (av (a+b,na+nb));
    Printf.printf "average forwarder chains length : %.2f\n%!" (av (j,r));
    Printf.printf "average request packages size   : %.2f\n%!" (av (p,r));
    Mutex.unlock m


let counter f x = fun y -> 
  Mutex.lock m; 
  x := f !x y; 
  Mutex.unlock m

let incr x () = x+1
let sum  (x,n) y = (x+y, n+1)
let sum2 (x,x',n) (y,y') = (x+y,x'+y',n+1)

let local_message = counter incr local_message
let net_message   = counter incr net_message  
let request       = counter sum2 request      
let completion    = counter incr completion   
let relocation    = counter incr relocation   
let forwarder     = counter sum  forwarder
let out_forwarder = counter sum  out_forwarder
let gc_forwarder  = counter incr gc_forwarder
let l_com         = counter incr l_com        
let l_in          = counter incr l_in         
let l_out         = counter incr l_out        
let l_open        = counter incr l_open       
