(*
 * Copyright (C) 2004 Damien Pous
 * 
 * This file is part of GcPan.
 * 
 * GcPan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * GcPan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GcPan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *)

(* Substitutions inside SA terms *)

open Ast


(* generic substitution of variables *)
let names' ?(abs=false) subst =
  if subst = [] then (fun p->p) else
    let s vv x = 
      try match List.assoc x vv with 
	| [Name n] -> n
	| _ -> failwith "bad substitution 1"
    with Not_found -> x
    in
    let abs_subst = 
      let replace v = 
	let rec aux acc = function
	  | [] -> v,acc
	  | (v',[Name n])::q when v=v' -> 
	      let n' = avar n in n',(v,[Name n'])::(List.rev_append acc q)
	  | vn::q -> aux (vn::acc) q
	in aux []
      in
      let rec aux vl vv = function
	| [] -> vv, List.rev vl
	| v::q -> let n,vv = replace v vv in aux (n::vl) vv q
      in aux []
    in
    let rec aux1 vv p = match p with
      | Pref(e,p)   -> prefix (aux1e vv e) (aux vv p)
      | Amb (v,p)   -> [Amb (s vv v, aux vv p)]
      | Amb'(h,v,p) -> [Amb'(h, s vv v, aux vv p)]
      | Send pl     -> [Send(List.map (auxe vv) pl)]
      | Rec (v,q)   -> [Rec(v, aux vv q)]
      | Nu  (v,q)   -> if List.mem_assoc v vv then [p] else [Nu (v, aux vv q)]
      | Rcv (vl,q) when abs -> let vv,vl = abs_subst vv vl in [Rcv(vl, aux vv q)]
      | Rcv (vl,q)  -> [Rcv(vl, aux (List.fold_left (fun x y -> List.remove_assoc y x) vv vl) q)]
      | Var _       -> [p]
    and aux vv = List.fold_left (fun a x -> par (aux1 vv x) a) zero
    and aux1e vv = function
      | Name x    -> (try List.assoc x vv with Not_found -> [Name x])
      | Capa(c,v) -> [Capa(c, s vv v)]
      | s         -> [s]
    and auxe vv l = List.rev (List.fold_left (fun a x -> List.rev_append (aux1e vv x) a) [] l)
    in
      aux subst

let names ?abs vars values = names' ?abs (List.combine vars values)


(* single name substitution *)
let name var value =
  let s x = if x=var then value else x in
  let rec aux1 p = match p with
    | Pref(e,p)   -> Pref(auxe e, aux p)
    | Amb (v,p)   -> Amb (s v, aux p)
    | Amb'(h,v,p) -> Amb'(h, s v, aux p)
    | Send pl     -> Send(List.map (List.map auxe) pl)
    | Rec (v,q)   -> Rec(v, aux q)
    | Nu  (v,q)   -> if v=var then p else Nu (v, aux q)
    | Rcv (vl,q)  -> if List.mem var vl then p else Rcv(vl, aux q)
    | Var _       -> p
  and aux l = List.rev_map aux1 l
  and auxe = function
    | Name v    -> Name (s v)
    | Capa(c,v) -> Capa(c, s v)
    | s         -> s
  in
    aux


(* substitute a recursion variable by its value *)
let process var process =
  let rec aux1 p = match p with
    | Amb (v,p)   -> [Amb (v, aux p)]
    | Amb'(h,v,p) -> [Amb'(h, v, aux p)]
    | Pref(l,p)   -> [Pref(l, aux p)]
    | Nu  (v,q)   -> [Nu (v, aux q)]
    | Rcv (v,q)   -> [Rcv(v, aux q)]
    | Rec (v,q)   -> if v=var then [p] else [Rec(v, aux q)]
    | Var v when v=var -> process
    | _           -> [p]
  and aux l = List.fold_left par zero (List.rev_map aux1 l)
  in
    aux

(* substitute a recursion variable by its (single) value *)
let process_t var process_t =
  let rec aux1 p = match p with
    | Amb (v,p)   -> Amb (v, aux p)
    | Amb'(h,v,p) -> Amb'(h, v, aux p)
    | Pref(l,p)   -> Pref(l, aux p)
    | Nu  (v,q)   -> Nu (v, aux q)
    | Rcv (v,q)   -> Rcv(v, aux q)
    | Rec (v,q)   -> if v=var then p else Rec(v, aux q)
    | Var v when v=var -> process_t
    | _           -> p
  and aux l = List.rev_map aux1 l
  in
    aux

